/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: check_user.c,v 1.29 2003/01/15 18:25:47 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <ctype.h>

/* lmlib includes */
#include <liblm.h>

/* Application includes */
#include "check_user.h"
#include "smtp.h"
#include "cdb.h"


static int(*_check_user_cleanup)(void);
static int(*_check_rcpt_user)(lm_string_t* addr, lm_string_t *pdotspam);

int msd_check_user_init_fs(void);
int msd_check_user_cleanup_fs(void);
int msd_check_rcpt_user_fs(lm_string_t* addr, lm_string_t* pdotspam);

int msd_check_user_init_db(void);
int msd_check_user_cleanup_db(void);
int msd_check_rcpt_user_db(lm_string_t* addr, lm_string_t* pdotspam);


int msd_check_user_init(void)
{
	int retval = 0;

	switch(config.compat_type)
	  {
	  case MSD_COMPAT_VPOP:
	  case MSD_COMPAT_QMAIL:
	    retval |= msd_check_user_init_fs();
	    _check_rcpt_user = msd_check_rcpt_user_fs;
	    _check_user_cleanup = msd_check_user_cleanup_fs;
	    break;
#ifdef USE_LM_DATABASE
	  case MSD_COMPAT_MAGIC:
	    retval |= msd_check_user_init_db();
	    _check_rcpt_user = msd_check_rcpt_user_db;
	    _check_user_cleanup = msd_check_user_cleanup_db;
	    break;
#endif	    
	  default:
	    /* Bad compat type */
	    SYSLOG((LOG_ERR, "Bad compat type"));
	    retval = -1;
	    break;
	  }
	
	/* Error during init, clear the function pointers */
	if(retval != 0)
	  {
	    _check_rcpt_user = NULL;
	    _check_user_cleanup = NULL;
	  }
	return retval;
}

int msd_check_user_cleanup(void)
{
  int retval = 0;

  LM_ASSERT(_check_user_cleanup);
  if(_check_user_cleanup != NULL)
    retval = _check_user_cleanup();
  _check_user_cleanup = NULL;
  _check_rcpt_user = NULL;
  return retval;
}

int msd_check_rcpt_user(lm_string_t* paddr, lm_string_t* pdotspam)
{
  char *buffer;
  LM_ASSERT(_check_rcpt_user);

  if(LM_STRING_LEN(*paddr) >= (sizeof("POSTMASTER")-1))
    {
      buffer = LM_STRING_BUFFER(*paddr);
      if(toupper(buffer[0]) == 'P' &&
	 toupper(buffer[1]) == 'O' &&
	 toupper(buffer[2]) == 'S' &&
	 toupper(buffer[3]) == 'T' &&
	 toupper(buffer[4]) == 'M' &&
	 toupper(buffer[5]) == 'A' &&
	 toupper(buffer[6]) == 'S' &&
	 toupper(buffer[7]) == 'T' &&
	 toupper(buffer[8]) == 'E' &&
	 toupper(buffer[9]) == 'R' &&
	 (buffer[10] == '\0' || buffer[10] == '@'))
	{
	  return 0;
	}
    }
	 
  return _check_rcpt_user(paddr, pdotspam);
}
