/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: config_msd.c,v 1.20 2003/02/14 00:22:53 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/* C includes */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <sys/stat.h>

/* Library includes */
#include <liblm.h>

/* Project includes */
#include "config_msd.h" 
#include "config_common.h"
#include "config_defaults.h"
#include "config_keys.h"
#include "config_spam_rule.h"
#include "config_virus_rule.h"

/* lobal configuration variables */
config_msd_t config;
lm_config_string_t newline;
lm_config_string_t emptystring;

#ifdef USE_LM_DATABASE
#include "db.h"
lm_string_t dbkeycol = NULL;
lm_string_t dbselectcol = NULL;
lm_string_t dbwhere = NULL;
#endif

/* Local function definitions */
static int config_msd_read(int argc, char **argv);

int config_msd_load(int argc, char **argv)
{
  int retval = 0;
  
  /* Initialize variables */
  memset(&config, 0, sizeof(config));
  config.helohost = NULL;
  config.fakehelohost = 0;
  retval |= LM_STRING_NEW(config.qmailcontroldir, MSD_QMAIL_CONTROL_DIR_DEFAULT);
  retval |= LM_STRING_NEW(config.qmailqueue, MSD_QMAIL_QUEUE_DEFAULT);
  config.loglevel = MSD_LOG_LEVEL_DEFAULT;
  retval |= LM_STRING_NEW(config.controldir, MSD_CONTROL_DIR_DEFAULT);
  config.maxlinelen = MSD_MAX_LINE_LENGTH_DEFAULT;
  config.maxhops = MSD_MAX_HOPS_DEFAULT;
  config.maxrcpt = MSD_MAX_RCPT_DEFAULT;
  config.rcptdelayat = MSD_RCPT_DELAY_AT_DEFAULT;
  config.rcptdelayinc = MSD_RCPT_DELAY_INC_DEFAULT;
  config.rcptdelaymax = MSD_RCPT_DELAY_MAX_DEFAULT;
  retval |= LM_STRING_NEW(config.check_vdomain_prog, MSD_CHECK_VDOMAIN_PROG_DEFAULT);
  config.check_valid_users = MSD_CHECK_VALID_USERS_DEFAULT;
  config.per_user_spam_enable = MSD_PER_USER_SPAM_ENABLE_DEFAULT;
  config.auth_enable = MSD_AUTH_ENABLE_DEFAULT;
  config.compat_type = MSD_COMPAT_TYPE_DEFAULT;
  config.max_smtp_cmds = MSD_MAX_SMTP_CMDS_DEFAULT;
  config.max_invalid_rcpt = MSD_MAX_INVALID_RCPT_DEFAULT;
  config.stray_newline_detection = MSD_STRAY_NEWLINE_DETECTION_DEFAULT;
  config.rfc_addr_only = MSD_RFC_ADDR_ONLY_DEFAULT;
  config.add_check_flags = MSD_ADD_CHECK_FLAGS_DEFAULT;
  retval |= LM_STRING_NEW(config.check_flag_sep, MSD_CHECK_FLAG_SEP_DEFAULT);
  retval |= LM_STRING_NEW(config.country_code_prog, MSD_COUNTRY_CODE_PROG_DEFAULT);
  retval |= LM_STRING_NEW(config.get_mx_rec_prog, MSD_GET_MX_REC_PROG_DEFAULT);
	
#ifdef USE_LM_DATABASE  
  retval |= LM_STRING_NEW(config.dbname, MSD_CONFIG_DBNAME_DEFAULT);
  retval |= LM_STRING_NEW(config.dbhost, MSD_CONFIG_DBHOST_DEFAULT);
  retval |= LM_STRING_NEW(config.dbuser, MSD_CONFIG_DBUSER_DEFAULT);
  retval |= LM_STRING_NEW(config.dbport, MSD_CONFIG_DBPORT_DEFAULT);
  retval |= LM_STRING_NEW(config.dbpwd, MSD_CONFIG_DBPWD_DEFAULT);
  retval |= LM_STRING_NEW(config.spamtable, MSD_CONFIG_SPAM_TABLE_DEFAULT);
  retval |= LM_STRING_NEW(config.virustable, MSD_CONFIG_VIRUS_TABLE_DEFAULT);
  retval |= LM_STRING_NEW(dbkeycol, "key");
  retval |= LM_STRING_NEW(dbselectcol, "value");
#endif /* USE_LM_DATABASE */
  
  retval |= LM_STRING_NEW(newline, "\n");
  retval |= LM_STRING_NEW(emptystring, "");

  if(!retval)
    retval = config_msd_read(argc, argv);
  return retval;
}

/* Clean up allocated memory */
int config_msd_cleanup(void)
{
  LM_STRING_FREE(config.qmailcontroldir);
  LM_STRING_FREE(config.qmailqueue);
  LM_STRING_FREE(config.controldir);
  LM_STRING_FREE(config.check_vdomain_prog);
  LM_STRING_FREE(config.helohost);
  LM_STRING_FREE(config.check_flag_sep);
  LM_STRING_FREE(config.country_code_prog);
  LM_STRING_FREE(config.get_mx_rec_prog);

  LM_STRING_FREE(newline);
  LM_STRING_FREE(emptystring);
#ifdef USE_LM_DATABASE  
  LM_STRING_FREE(dbkeycol);
  LM_STRING_FREE(dbselectcol);
  LM_STRING_FREE(config.dbname);
  LM_STRING_FREE(config.dbhost);
  LM_STRING_FREE(config.dbuser);
  LM_STRING_FREE(config.dbport);
  LM_STRING_FREE(config.dbpwd);
  LM_STRING_FREE(config.conftable);
  LM_STRING_FREE(config.spamtable);
  LM_STRING_FREE(config.virustable);
  db_cleanup();
#endif  
  return 0;
}

/* Perform checks on variables to make sure the values stored in them
   make sense, else print error message and return error */
int config_msd_verify(config_msd_t config)
{
  struct stat buf;

  /* RFC Says we have to have at least 100 recipients */
  if(config.maxrcpt < MSD_MAX_RCPT_DEFAULT ||
     config.maxrcpt < 0)
    {
      SYSLOG((LOG_ERR, "maxrcpt setting was bad. setting to default: %ld.", MSD_MAX_RCPT_DEFAULT));
      config.maxrcpt = MSD_MAX_RCPT_DEFAULT;
    }
  
  /* Check that the qmailqueue program exists */
  if(lstat(LM_STRING_BUFFER(config.qmailqueue), &buf) != 0)
    {
      SYSLOG((LOG_ERR, "qmail-queue program does not exist! setting to default: %s", MSD_QMAIL_QUEUE_DEFAULT));
      LM_STRING_FREE(config.qmailqueue);
      if(LM_STRING_NEW(config.qmailqueue, MSD_QMAIL_QUEUE_DEFAULT))
	return -1;
    }
	
  /* Check that vdomainprog exists and is executable */
  if(lstat(LM_STRING_BUFFER(config.check_vdomain_prog), &buf) != 0)
    {
      SYSLOG((LOG_ERR, "virtual domain checking program does not exist! setting to default: %s", MSD_CHECK_VDOMAIN_PROG_DEFAULT));
      LM_STRING_FREE(config.check_vdomain_prog);
      if(LM_STRING_NEW(config.check_vdomain_prog, MSD_CHECK_VDOMAIN_PROG_DEFAULT))
	return -1;
    }
  
  /* Check that country code prog exists and is executable */
  if(lstat(LM_STRING_BUFFER(config.country_code_prog), &buf) != 0)
    {
      SYSLOG((LOG_ERR, "country code lookup program does not exist! setting to default: %s", MSD_COUNTRY_CODE_PROG_DEFAULT));
      LM_STRING_FREE(config.country_code_prog);
      if(LM_STRING_NEW(config.country_code_prog, MSD_COUNTRY_CODE_PROG_DEFAULT))
	return -1;
    }

  /* Check that get mx record prog exists and is executable */
  if(lstat(LM_STRING_BUFFER(config.get_mx_rec_prog), &buf) != 0)
    {
      SYSLOG((LOG_ERR, "mx record lookup program does not exist! setting to default: %s", MSD_GET_MX_REC_PROG_DEFAULT));
      LM_STRING_FREE(config.get_mx_rec_prog);
      if(LM_STRING_NEW(config.get_mx_rec_prog, MSD_GET_MX_REC_PROG_DEFAULT))
	return -1;
    }

  /* Check that max hops is > 0 */
  if(config.maxhops < 0)
    {
      SYSLOG((LOG_ERR, "maxhops value was bad. setting to default: %d", MSD_MAX_HOPS_DEFAULT));
      config.maxhops = MSD_MAX_HOPS_DEFAULT;
    }

  return 0;
}

#define READ_NUMBER(NAME, LNAME) \
if(retval == 0) \
{ \
  if(type == READ_CONTROL) \
    retval |= config_read_number(MSD_CONTROL_##NAME, \
				 &config.LNAME, config.LNAME); \
  else \
    retval |= config_read_number(MSD_ENV_##NAME, \
				 &config.LNAME, config.LNAME); \
}
#define READ_STRING(NAME, LNAME) \
if(retval == 0) \
{ \
  if(type == READ_CONTROL) \
    retval |= config_read_string(MSD_CONTROL_##NAME, \
				 &config.LNAME, config.LNAME); \
  else \
    retval |= config_read_string(MSD_ENV_##NAME, \
				 &config.LNAME, config.LNAME); \
}

#define READ_CONTROL 0
#define READ_ENV     1
static int config_read_variables(int type)
{
  int retval = 0;
  
  LM_ASSERT(type == READ_CONTROL || type == READ_ENV);
  
  if(type == READ_CONTROL)
    {
      /* Open the env config */
      if(LM_CONFIG_CONTROL_SET_PARAM(config.controldir))
	{
	  SYSLOG((LOG_ERR, "Could not open control directory %s!",
		  LM_STRING_BUFFER(config.controldir))); 
	  return 0;
	}
    }
  else if (type == READ_ENV)
    {
      /* Open the env config */
      if(LM_CONFIG_ENV_SET_PARAM)
	{
	  SYSLOG((LOG_ERR, "Could not open environment config."));
	  return 0;
	}
    }
  
  /* Read the lmsmptd debug level env variable */
  READ_NUMBER(LOG_LEVEL, loglevel);
  READ_NUMBER(AUTH_ENABLE, auth_enable);
  READ_NUMBER(COMPAT_TYPE, compat_type);
  READ_STRING(QMAIL_QUEUE, qmailqueue);
  READ_STRING(QMAIL_CONTROL_DIR, qmailcontroldir);
  READ_NUMBER(MAX_LINE_LENGTH, maxlinelen);
  READ_NUMBER(MAX_HOPS, maxhops);
  READ_NUMBER(MAX_RCPT, maxrcpt);
  READ_NUMBER(RCPT_DELAY_AT, rcptdelayat);
  READ_NUMBER(RCPT_DELAY_INC, rcptdelayinc);
  READ_NUMBER(RCPT_DELAY_MAX, rcptdelaymax);
  READ_NUMBER(CHECK_VALID_USERS, check_valid_users);
  READ_NUMBER(MAX_SMTP_CMDS, max_smtp_cmds);
  READ_NUMBER(MAX_INVALID_RCPT, max_invalid_rcpt);
  READ_NUMBER(STRAY_NEWLINE_DETECTION, stray_newline_detection);
  
  /* Read if we should check for superuser owned spam enable files in the user's home directory */
  READ_NUMBER(PER_USER_SPAM_ENABLE, per_user_spam_enable);
  READ_STRING(VDOMAIN_CHECKUSER_PROG, check_vdomain_prog);
  READ_NUMBER(RFC_ADDR_ONLY, rfc_addr_only);
  READ_NUMBER(ADD_CHECK_FLAGS, add_check_flags);
  READ_STRING(CHECK_FLAG_SEP, check_flag_sep);
  READ_STRING(CONTROL_DIR, controldir); /* not really needed but consistent */
  READ_STRING(COUNTRY_CODE_PROG, country_code_prog);
  READ_STRING(GET_MX_REC_PROG, get_mx_rec_prog);

#ifdef USE_LM_DATABASE  
  if(config.compat_type == MSD_COMPAT_MAGIC)
    {
      /* Read the dbname */
      READ_STRING(DBNAME, dbname);
      READ_STRING(DBHOST, dbhost);
      READ_STRING(DBPORT, dbport);
      READ_STRING(DBUSER, dbuser);
      READ_STRING(DBPWD, dbpwd);
      READ_STRING(SPAM_TABLE, spamtable);
      READ_STRING(VIRUS_TABLE, virustable);
    }
#endif
  
  /*** Close the env config ***/
  LM_CONFIG_CLOSE;
  
  return retval;
}

static int config_msd_read(int argc, char **argv)
{
  if(config_read_variables(READ_CONTROL) ||
     config_read_variables(READ_ENV))
    return -1;

  /*** Load the program config from proper location depending upon 
       compatibility given by env config ***/
  switch(config.compat_type)
    {
    case MSD_COMPAT_QMAIL:
    case MSD_COMPAT_VPOP:
      if(LM_CONFIG_CONTROL_SET_PARAM(config.controldir))
	{
	  SYSLOG((LOG_ERR, "could not open control directory: %s", LM_STRING_BUFFER(config.controldir)));
	}

      /* Load the global spam rules from the control file directory */
      config_spam_rule_read(NULL, &global_spam_rules);
      config_virus_rule_read(NULL, &global_virus_rules);
      
      LM_CONFIG_CLOSE;
      break;

#ifdef USE_LM_DATABASE      
    case MSD_COMPAT_MAGIC:
      /* Open the database connetion */
      if(db_init() == 0)
	{
	  /* TODO Change this to the global rule name */
	  LM_STRING_NEW(dbwhere, "and name='email'");
	  if(LM_CONFIG_DB_SET_PARAM(db, config.spamtable,
				    dbselectcol, dbkeycol, dbwhere))
	    {
	      SYSLOG((LOG_ERR, "error setting up virus db paramters."));
	    }
	  
	  /* Load the global spam rules from the control file directory */
	  config_spam_rule_read(NULL, &global_spam_rules);
	  LM_CONFIG_CLOSE;
	  
	  if(LM_CONFIG_DB_SET_PARAM(db, config.virustable,
				    dbselectcol, dbkeycol, dbwhere))
	    {
	      SYSLOG((LOG_ERR, "error setting up virus db paramters."));
	    }
	  
	  /* Load the global spam rules from the control file directory */
	  config_virus_rule_read(NULL, &global_virus_rules);
	  LM_CONFIG_CLOSE;
	  
	  LM_STRING_FREE(dbwhere);
	}
      else
	{
	  SYSLOG((LOG_WARNING, "Could not connect to database"));
	  
	  /* Set these up to not do anything since we can't do these
	     without a db connection */
	  global_spam_rules.spam_check = 0;
	  config.check_valid_users = 0;
	}
      break;
#endif      
		
    default:
      SYSLOG((LOG_ERR, "Invalid value for MSD_COMPAT_TYPE! Could not load config"));
      break;
    }
	
  /* Free up remaining memory */
  return config_msd_verify(config);
}
