#include <stdio.h>

#include <liblm.h>

int main(int argc, char **argv)
{
  lm_string_t controldir;
  lm_config_string_t config_string;
  lm_config_string_t config_string_default;
  lm_config_number_t config_number;
  lm_config_number_t config_number_default;
  lm_config_bool_t config_bool;
  lm_config_bool_t config_bool_default;
  lm_config_string_t string_key;
  lm_config_string_t number_key;
  lm_config_string_t bool_key;
  

  /* Initialize pointers to null */
  controldir = NULL;
  string_key = number_key = bool_key = NULL;
  config_string = NULL;
  config_number = config_bool = 0;
  
  LM_LOG_INIT("./lmconfig-test.log");
  
  /* THis will attempt to setup lmconfig to use control files
     and return an error if it can't */
  LM_STRING_NEW(controldir, "./control/");
  if(LM_CONFIG_CONTROL_SET_PARAM(controldir))
    {
      LM_LOG_WARN("lmconfig-test", "Control directory ./control does not exist!");
    }

  /* Allocate the keys to access the configuration variables */
  LM_STRING_NEW(string_key, "configstring");
  LM_STRING_NEW(number_key, "confignumber");
  LM_STRING_NEW(bool_key, "configbool");

  config_string_default = NULL;
  LM_STRING_NEW(config_string_default, "default value");
  config_number_default = -1;
  config_bool_default = 0;
  
  /* Read the configuration variables from the config */
  LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);
  LM_CONFIG_READ_NUMBER(number_key, config_number, config_number_default);
  LM_CONFIG_READ_BOOL(bool_key, config_bool, config_bool_default);
  
  /* Read the configuration variables from the config */
  LM_CONFIG_WRITE_STRING(string_key, config_string);
  LM_CONFIG_WRITE_NUMBER(number_key, config_number);
  LM_CONFIG_WRITE_BOOL(bool_key, config_bool);
  
  /* Close the configuration when done */
  LM_CONFIG_CLOSE;

  /* Free allocated memory */
  LM_STRING_FREE(controldir);
  LM_STRING_FREE(string_key);
  LM_STRING_FREE(number_key);
  LM_STRING_FREE(bool_key);
  
  /* Test environment config type */
  LM_CONFIG_ENV_SET_PARAM;

  LM_STRING_NEW(string_key, "USER");
  LM_STRING_FREE(config_string_default);
  LM_STRING_NEW(config_string_default, "username unknown");
  LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);

  LM_STRING_FREE(string_key);
  LM_STRING_NEW(string_key, "UNKNOWN");
  LM_STRING_FREE(config_string_default);
  LM_STRING_NEW(config_string_default, "default value");
  LM_CONFIG_READ_STRING(string_key, config_string, config_string_default);

  LM_CONFIG_CLOSE;

  LM_STRING_FREE(string_key);
  LM_STRING_FREE(config_string_default);

  /* Free this when it gets filled in by config */
  LM_STRING_FREE(config_string);

  LM_MEMDUMP(stdout);

  
  return 0;
}
