/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: pgquery.c,v 1.2.2.1 2003/02/14 19:20:04 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */
#include <stdlib.h>

#include <getopt.h>
#include <postgresql/libpq-fe.h>
#include <postgresql/libpgeasy.h>

char *fieldsep = ",";
char *query = NULL;
char *dbuser, *dbname, *dbhost, *dbpwd, *dbport;

int main(int argc, char **argv)
{
  char buffer[4096];
  PGconn *db = NULL;
  PGresult *res = NULL;
  int i, j, c;
  int len = 0;
  int retval = 1;

  buffer[0] = '\0';
  dbname = getenv("PGDATABASE");
  dbhost = getenv("PGHOST");
  dbport = getenv("PGPORT");
  dbuser = getenv("PGUSER");
  dbpwd = getenv("PGPASSWORD");
  
  while((c = getopt(argc, argv, "u:n:p:h:P:s:")) != -1)
    {
      switch(c)
	{
	case 'u':
	  dbuser = optarg;
	  break;
	case 'n':
	  dbname = optarg;
	  break;
	case 'p':
	  dbpwd = optarg;
	  break;
	case 'h':
	  dbhost = optarg;
	  break;
	case 'P':
	  dbport = optarg;
	  break; 
	case 's':
	  fieldsep = optarg;
	  break;
	case '?':
	  break;
	case ':':
	  break;
	}
    }

  len = 0;
  dbuser ? len += snprintf(buffer+len, sizeof(buffer), "user=%s ", dbuser) : 0;
  dbname ? len += snprintf(buffer+len, sizeof(buffer), "dbname=%s ", dbname) : 0;
  dbhost ? len += snprintf(buffer+len, sizeof(buffer), "host=%s ", dbhost) : 0;
  dbpwd ? len += snprintf(buffer+len, sizeof(buffer), "password=%s ", dbpwd) : 0;
  dbport ? len += snprintf(buffer+len, sizeof(buffer), "port=%s ", dbport) : 0;

  if(optind >= argc)
    {
      printf("Usage: %s -unphPs 'query' (use ' or \" quotes around the query)\n", argv[0]);
      return 1;
    }
  query = argv[optind];

  db = PQconnectdb(buffer);
  if(db == NULL ||
     PQstatus(db) == CONNECTION_BAD)
    {
      fprintf(stderr, "%s", PQerrorMessage(db));
      return 1;
    }

  res = PQexec(db, query);
  if(res == NULL ||
     PQresultStatus(res) == PGRES_BAD_RESPONSE ||
     PQresultStatus(res) == PGRES_NONFATAL_ERROR ||
     PQresultStatus(res) == PGRES_FATAL_ERROR)
    {
      if(db != NULL)
	fprintf(stderr, "%s", PQerrorMessage(db));
    }

  switch(PQresultStatus(res))
    {
    case PGRES_COMMAND_OK:
      /* no tuples or fields so this will fall through to set retval */
    case PGRES_TUPLES_OK:
      {
	int ntuples = PQntuples(res);
	int nfields = PQnfields(res);
	
	for(i=0;i<ntuples;i++)
	  {
	    len = 0;
	    for(j=0;j<nfields;j++)
	      {
		char *value = PQgetvalue(res, i, j);
		if(value)
		  {
		    len += snprintf(buffer+len, sizeof(buffer),
 				    "%s%s", value, fieldsep);
		  }
	      }
	    if(len > 0)
	      {
		/* get rid of the trailing seperator */
		buffer[--len] = '\0';
		printf("%s\n", buffer);
	      }
	  }
	retval = 0;
	break;
      }
      
    case PGRES_EMPTY_QUERY:
    case PGRES_COPY_OUT:
    case PGRES_COPY_IN:
    case PGRES_BAD_RESPONSE:
    case PGRES_NONFATAL_ERROR:
    case PGRES_FATAL_ERROR:
      fprintf(stderr, "RESULT STATUS: %s\n", PQresStatus(PQresultStatus(res)));
      retval = 1;
      break;
    }	
  
  PQclear(res);
  PQfinish(db);
  exit(retval);
}
