/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: config_virus_rule.c,v 1.2 2003/02/04 22:35:02 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

#include <string.h>

#include "config_msd.h"
#include "config_common.h"
#include "config_virus_rule.h"
#include "config_misc_rule.h"
#include "db.h"

config_virus_rule_t global_virus_rules;

int config_virus_rule_init_defaults(config_virus_rule_t *rules);
int config_virus_rule_init_defaults_from_globals(config_virus_rule_t* rules, config_virus_rule_t* global_rules);
int config_virus_rule_load_lists(config_virus_rule_t *rules, config_virus_rule_t *global_rules);
int config_virus_rule_load_variables(config_virus_rule_t *rules);


#ifdef USE_LM_DATABASE
int config_virus_rule_load_db(lm_string_t addr, 
			     config_virus_rule_t *rules)
{
  int retval;
  char buffer[1024];
  char *escuser = NULL;
  char *escdomain = NULL;
	
  msd_db_get_user_domain(&escuser, &escdomain, addr);
	
  /* Setup the database paramters for the user */
  snprintf(buffer, sizeof(buffer),
	   /*"and name='%s' and domain='%s'",*/
	   "and name=actual_name('%s','%s',4) and domain='%s'",
	   escuser, escdomain, escdomain);
  LM_STRING_NEW(dbwhere, buffer);
  retval = LM_CONFIG_DB_SET_PARAM(db, config.virustable,
				  dbselectcol, dbkeycol, dbwhere);
  LM_STRING_FREE(dbwhere);
	
  /* Load the virus rules for the set config */
  if(retval == 0)
    {
      /* load the user rules and append global rules as per user configuration */
      retval = config_virus_rule_read(&global_virus_rules, rules);
    }
  /* Close the configuration that was opened */
  LM_CONFIG_CLOSE;

  LM_FREE(escuser);
  LM_FREE(escdomain);

  return retval;
}
#endif

/* This is a bit of a hack so that we don't have to have two spamdir type 
   directories on a non-database system.  All the virus and spam rules can 
   be contained in the same place.  There is a conflict with the
   all_glboal_rules control file, but that shouldn't be a problem since all 
   does is control rule inheritance */
int config_virus_rule_load_spamdir(config_virus_rule_t *rules, 
				  lm_string_t dotspam)
{
  int retval;
  
  /* Stat the dotvirus file 
   * The check user functions return a trailing / on filenames if they 
   * happen to be a directory so we don't have to stat the file again
   */
  /* If the file is a directory */
  if(LM_STRING_BUFFER(dotspam)[LM_STRING_LEN(dotspam)-1] == '/')
    {
      /* load user rules as control files */
      SYSLOG((LOG_INFO, "Loading spam rule control files from %s", LM_STRING_BUFFER(dotspam)));
      retval = LM_CONFIG_CONTROL_SET_PARAM(dotspam);
    }
  else
    {
      SYSLOG((LOG_WARNING, "This version supports spam control directories only! Spam rules not loaded."));
      return -1;
    }
  
  /* Load the spam rules for the set config */
  if(retval == 0)
    {
      /* load the user rules and append global rules as per user configuration */
      retval = config_virus_rule_read(&global_virus_rules, rules);
    }
  /* Close the configuration that was opened */
  LM_CONFIG_CLOSE;
  
  return retval;
}

int config_virus_rule_load(lm_string_t addr, 
		config_virus_rule_t *rules, lm_string_t dotspam)
{
  int retval = 0;
  
  LM_ASSERT(rules != NULL);
  switch(config.compat_type)
    {
    case MSD_COMPAT_VPOP:
    case MSD_COMPAT_QMAIL:
      LM_ASSERT(dotspam != NULL);
      retval = config_virus_rule_load_spamdir(rules, dotspam);
      break;
#ifdef USE_LM_DATABASE      
    case MSD_COMPAT_MAGIC:
      LM_ASSERT(addr != NULL);
      if(db != NULL)
	retval = config_virus_rule_load_db(addr, rules);
      else
	retval = -1;
      break;
#endif
    default:
      SYSLOG((LOG_ERR, "Invalid compat type!"));
      retval = -1;
    }
  return retval;
}

int config_virus_rule_free(config_virus_rule_t *ruledata)
{
  return 0;  
}

int config_virus_rule_verify(config_virus_rule_t *rules)
{
  return 0;
}

int config_virus_rule_read(config_virus_rule_t* global_rules, config_virus_rule_t *rules)
{
  int retval = 0;

  LM_ASSERT(rules != NULL);
  /* Don't really need this since we know we will be filling in
     all of the variables in the structure */
  memset(rules, 0, sizeof(*rules));

  /* Load the special key to specify to use all global rules so
     we know how to initialize the structure */
  rules->all_global_rules = 0;
  if(global_rules != NULL)
    {
      LOAD_BOOL(VIRUS, (NODEPS), ALL_GLOBAL_RULES, all_global_rules);
    }

  /* If the user specified to use all global rules, set the default values to those
   * in the globals, else set to the ones specified in the documentation
   * This only applies to boolean rules here.  The lists are also inherited later in the code.
   */
  if(rules->all_global_rules == 0)
    retval |= config_virus_rule_init_defaults(rules);
  else
    retval |= config_virus_rule_init_defaults_from_globals(rules, global_rules);

  /* Load the boolean variables */
  if(retval == 0)
    retval |= config_virus_rule_load_variables(rules);

  /* Verify that the configuration is ok */
  if(retval == 0)
    retval |= config_virus_rule_verify(rules);

  return retval;
}

int config_virus_rule_init_defaults_from_globals(config_virus_rule_t* rules, config_virus_rule_t* global_rules)
{
  int retval = 0;

  rules->all_global_rules = global_rules->all_global_rules;
  rules->virus_check = global_rules->virus_check;
  return retval;
}

int config_virus_rule_init_defaults(config_virus_rule_t* rules)
{
  int retval = 0;

  rules->all_global_rules = VIRUS_CONTROL_ALL_GLOBAL_RULES_DEFAULT;
  rules->virus_check = VIRUS_CONTROL_VIRUS_CHECK_DEFAULT;
  
  return retval;
}

int config_virus_rule_load_variables(config_virus_rule_t *rules)
{
  int retval = 0;

  LOAD_BOOL(VIRUS, (NODEPS),
	    ALL_GLOBAL_RULES, all_global_rules);
  LOAD_BOOL(VIRUS, (NODEPS),
	    VIRUS_CHECK, virus_check);
  return retval;
}

int config_virus_rule_load_lists(config_virus_rule_t *rules, config_virus_rule_t *global_rules)
{
  int retval = 0;
  return retval;
}
